"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a, _b, _c, _d;
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
const fs = require("fs");
src_1.LogService.setLogger(new src_1.RichConsoleLogger());
src_1.LogService.setLevel(src_1.LogLevel.TRACE);
src_1.LogService.muteModule("Metrics");
src_1.LogService.trace = src_1.LogService.debug;
let creds = null;
try {
    creds = require("../../examples/storage/encryption_appservice.creds.json");
}
catch (e) {
    // ignore
}
const dmTarget = (_a = creds === null || creds === void 0 ? void 0 : creds['dmTarget']) !== null && _a !== void 0 ? _a : "@admin:localhost";
const homeserverUrl = (_b = creds === null || creds === void 0 ? void 0 : creds['homeserverUrl']) !== null && _b !== void 0 ? _b : "http://localhost:8008";
const storage = new src_1.SimpleFsStorageProvider("./examples/storage/encryption_appservice.json");
const crypto = new src_1.RustSdkAppserviceCryptoStorageProvider("./examples/storage/encryption_appservice_sled");
const worksImage = fs.readFileSync("./examples/static/it-works.png");
const registration = {
    as_token: (_c = creds === null || creds === void 0 ? void 0 : creds['asToken']) !== null && _c !== void 0 ? _c : "change_me",
    hs_token: (_d = creds === null || creds === void 0 ? void 0 : creds['hsToken']) !== null && _d !== void 0 ? _d : "change_me",
    sender_localpart: "crypto_test_appservice_rust3",
    namespaces: {
        users: [{
                regex: "@crypto.*:localhost",
                exclusive: true,
            }],
        rooms: [],
        aliases: [],
    },
    "de.sorunome.msc2409.push_ephemeral": true,
};
const options = {
    bindAddress: "0.0.0.0",
    port: 9000,
    homeserverName: "localhost",
    homeserverUrl: homeserverUrl,
    storage: storage,
    registration: registration,
    joinStrategy: new src_1.SimpleRetryJoinStrategy(),
    cryptoStorage: crypto,
    intentOptions: {
        encryption: true,
    },
};
const appservice = new src_1.Appservice(options);
// const bot = appservice.botIntent;
const bot = appservice.getIntentForUserId("@crypto_nondefault_test3:localhost");
(function () {
    return __awaiter(this, void 0, void 0, function* () {
        yield bot.enableEncryption();
        let encryptedRoomId;
        const joinedRooms = yield bot.underlyingClient.getJoinedRooms();
        for (const roomId of joinedRooms) {
            if (yield bot.underlyingClient.crypto.isRoomEncrypted(roomId)) {
                const members = yield bot.underlyingClient.getJoinedRoomMembers(roomId);
                if (members.length >= 2) {
                    encryptedRoomId = roomId;
                    break;
                }
            }
        }
        if (!encryptedRoomId) {
            encryptedRoomId = yield bot.underlyingClient.createRoom({
                invite: [dmTarget],
                is_direct: true,
                visibility: "private",
                preset: "trusted_private_chat",
                initial_state: [
                    { type: "m.room.encryption", state_key: "", content: { algorithm: src_1.EncryptionAlgorithm.MegolmV1AesSha2 } },
                    { type: "m.room.guest_access", state_key: "", content: { guest_access: "can_join" } },
                ],
            });
        }
        appservice.on("room.failed_decryption", (roomId, event, e) => __awaiter(this, void 0, void 0, function* () {
            src_1.LogService.error("index", `Failed to decrypt ${roomId} ${event['event_id']} because `, e);
        }));
        appservice.on("room.message", (roomId, event) => __awaiter(this, void 0, void 0, function* () {
            if (roomId !== encryptedRoomId)
                return;
            const message = new src_1.MessageEvent(event);
            if (message.sender === bot.userId && message.messageType === "m.notice") {
                // yay, we decrypted our own message. Communicate that back for testing purposes.
                const encrypted = yield bot.underlyingClient.crypto.encryptMedia(Buffer.from(worksImage));
                const mxc = yield bot.underlyingClient.uploadContent(encrypted.buffer);
                yield bot.underlyingClient.sendMessage(roomId, {
                    msgtype: "m.image",
                    body: "it-works.png",
                    info: {
                        // XXX: We know these details, so have hardcoded them.
                        w: 256,
                        h: 256,
                        mimetype: "image/png",
                        size: worksImage.length,
                    },
                    file: Object.assign({ url: mxc }, encrypted.file),
                });
                return;
            }
            if (message.messageType === "m.image") {
                const fileEvent = new src_1.MessageEvent(message.raw);
                const decrypted = yield bot.underlyingClient.crypto.decryptMedia(fileEvent.content.file);
                fs.writeFileSync('./examples/storage/decrypted.png', decrypted);
                yield bot.underlyingClient.unstableApis.addReactionToEvent(roomId, fileEvent.eventId, 'Decrypted');
                return;
            }
            if (message.messageType !== "m.text")
                return;
            if (message.textBody.startsWith("!ping")) {
                yield bot.underlyingClient.replyNotice(roomId, event, "Pong");
            }
        }));
        src_1.LogService.info("index", "Starting appservice...");
        yield appservice.begin();
    });
})();
