"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a, _b, _c;
Object.defineProperty(exports, "__esModule", { value: true });
const src_1 = require("../src");
const fs = require("fs");
src_1.LogService.setLogger(new src_1.RichConsoleLogger());
src_1.LogService.setLevel(src_1.LogLevel.TRACE);
src_1.LogService.muteModule("Metrics");
src_1.LogService.trace = src_1.LogService.debug;
let creds = null;
try {
    creds = require("../../examples/storage/encryption_bot.creds.json");
}
catch (e) {
    // ignore
}
const dmTarget = (_a = creds === null || creds === void 0 ? void 0 : creds['dmTarget']) !== null && _a !== void 0 ? _a : "@admin:localhost";
const homeserverUrl = (_b = creds === null || creds === void 0 ? void 0 : creds['homeserverUrl']) !== null && _b !== void 0 ? _b : "http://localhost:8008";
const accessToken = (_c = creds === null || creds === void 0 ? void 0 : creds['accessToken']) !== null && _c !== void 0 ? _c : 'YOUR_TOKEN';
const storage = new src_1.SimpleFsStorageProvider("./examples/storage/encryption_bot.json");
const crypto = new src_1.RustSdkCryptoStorageProvider("./examples/storage/encryption_bot_sled");
const worksImage = fs.readFileSync("./examples/static/it-works.png");
const client = new src_1.MatrixClient(homeserverUrl, accessToken, storage, crypto);
(function () {
    return __awaiter(this, void 0, void 0, function* () {
        let encryptedRoomId;
        const joinedRooms = yield client.getJoinedRooms();
        yield client.crypto.prepare(joinedRooms); // init crypto because we're doing things before the client is started
        for (const roomId of joinedRooms) {
            if (yield client.crypto.isRoomEncrypted(roomId)) {
                encryptedRoomId = roomId;
                break;
            }
        }
        if (!encryptedRoomId) {
            encryptedRoomId = yield client.createRoom({
                invite: [dmTarget],
                is_direct: true,
                visibility: "private",
                preset: "trusted_private_chat",
                initial_state: [
                    { type: "m.room.encryption", state_key: "", content: { algorithm: src_1.EncryptionAlgorithm.MegolmV1AesSha2 } },
                    { type: "m.room.guest_access", state_key: "", content: { guest_access: "can_join" } },
                ],
            });
        }
        client.on("room.failed_decryption", (roomId, event, e) => __awaiter(this, void 0, void 0, function* () {
            src_1.LogService.error("index", `Failed to decrypt ${roomId} ${event['event_id']} because `, e);
        }));
        client.on("room.message", (roomId, event) => __awaiter(this, void 0, void 0, function* () {
            if (roomId !== encryptedRoomId)
                return;
            const message = new src_1.MessageEvent(event);
            if (message.sender === (yield client.getUserId()) && message.messageType === "m.notice") {
                // yay, we decrypted our own message. Communicate that back for testing purposes.
                const encrypted = yield client.crypto.encryptMedia(Buffer.from(worksImage));
                const mxc = yield client.uploadContent(encrypted.buffer);
                yield client.sendMessage(roomId, {
                    msgtype: "m.image",
                    body: "it-works.png",
                    info: {
                        // XXX: We know these details, so have hardcoded them.
                        w: 256,
                        h: 256,
                        mimetype: "image/png",
                        size: worksImage.length,
                    },
                    file: Object.assign({ url: mxc }, encrypted.file),
                });
                return;
            }
            if (message.messageType === "m.image") {
                const fileEvent = new src_1.MessageEvent(message.raw);
                const decrypted = yield client.crypto.decryptMedia(fileEvent.content.file);
                fs.writeFileSync('./examples/storage/decrypted.png', decrypted);
                yield client.unstableApis.addReactionToEvent(roomId, fileEvent.eventId, 'Decrypted');
                return;
            }
            if (message.messageType !== "m.text")
                return;
            if (message.textBody.startsWith("!ping")) {
                yield client.replyNotice(roomId, event, "Pong");
            }
        }));
        src_1.LogService.info("index", "Starting bot...");
        yield client.start();
    });
})();
