"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncV3MatrixClient = void 0;
const MatrixClient_1 = require("./MatrixClient");
const crypto = require("crypto");
const LogService_1 = require("./logging/LogService");
/**
 * A MatrixClient class which attempts to use Sync V3 instead of the normal sync protocol
 * on the server.
 *
 * This class is considered <b>UNSTABLE</b> and may be removed at any time. This class will
 * not check for server support before attempting to use it.
 */
class SyncV3MatrixClient extends MatrixClient_1.MatrixClient {
    /**
     Creates a new matrix client
     @param {string} homeserverUrl The homeserver's client-server API URL
     @param {string} accessToken The access token for the homeserver
     @param {IStorageProvider} storage The storage provider to use.
     */
    constructor(homeserverUrl, accessToken, storage) {
        super(homeserverUrl, accessToken, storage);
    }
    startSyncInternal() {
        return __awaiter(this, void 0, void 0, function* () {
            this.syncSessionId = yield this.storageProvider.readValue("sync_v3_session");
            if (!this.syncSessionId) {
                this.syncSessionId = crypto.randomBytes(16).toString('hex');
                yield this.storageProvider.storeValue("sync_v3_session", this.syncSessionId);
            }
            this.loopSync();
        });
    }
    loopSync() {
        const promiseWhile = () => __awaiter(this, void 0, void 0, function* () {
            if (this.stopSyncing) {
                LogService_1.LogService.info("MatrixClientLite", "Client stop requested - stopping sync");
                return;
            }
            try {
                const response = yield this.doSyncV3();
                yield this.processSyncV3(response);
            }
            catch (e) {
                LogService_1.LogService.error("MatrixClientLite", "Error handling sync " + LogService_1.extractRequestError(e));
                const backoffTime = 5000 + Math.random() * (15000 - 5000); // TODO: de-hardcode values. SYNC_BACKOFF_MIN_MS SYNC_BACKOFF_MAX_MS
                LogService_1.LogService.info("MatrixClientLite", `Backing off for ${backoffTime}ms`);
                yield new Promise((r) => setTimeout(r, backoffTime));
            }
            return promiseWhile();
        });
        promiseWhile(); // start loop async
    }
    processSyncV3(sync) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('@@', sync);
        });
    }
    doSyncV3() {
        return this.doRequest("POST", "/_matrix/client/unstable/org.matrix.msc3575/sync", {}, {
            session_id: this.syncSessionId,
            lists: [{
                    // First list: default catch-all for joined rooms.
                    // We want all rooms
                    //rooms: [[0, 99]],
                    rooms: [[0, Number.MAX_SAFE_INTEGER]],
                    // We don't care about sort
                    //sort: ["by_notification_count"],
                    // We don't have any required state, so tell the server that
                    required_state: [],
                    // We don't want any previous timeline state to avoid confusing bots
                    timeline_limit: 0,
                }, {
                    // Second list: invites only
                    filters: {
                        is_invite: true,
                    },
                    priority: 1,
                }],
            // TODO: Support extensions for crypto and such
        });
    }
}
exports.SyncV3MatrixClient = SyncV3MatrixClient;
