"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NamespacingPostgresCryptoStorageProvider = void 0;
const pgPromise = require("pg-promise");
const DEFAULT_NAMESPACE = "default";
// Init library
const psql = pgPromise();
/**
 * PostgreSQL crypto storage provider for appservices and clients. Requires `pg-promise` package to be installed.
 * @category Storage providers
 */
class NamespacingPostgresCryptoStorageProvider {
    /**
     * Combined constructor implementation.
     */
    constructor(cnOrNamespace, parentOrSecureStorage) {
        this.namespace = DEFAULT_NAMESPACE;
        this.kvUpsert = "INSERT INTO kv (ns, name, value) VALUES ($[ns], $[name], $[value]) ON CONFLICT (ns, name) DO UPDATE SET value = $[value]";
        this.kvSelect = "SELECT ns, name, value FROM kv WHERE name = $[name] AND ns = $[ns]";
        this.roomUpsert = "INSERT INTO rooms (room_id, config) VALUES ($[roomId], $[config]) ON CONFLICT (room_id) DO UPDATE SET config = $[config]";
        this.roomSelect = "SELECT room_id, config FROM rooms WHERE room_id = $[roomId]";
        this.userUpsert = "INSERT INTO users (user_id, outdated) VALUES ($[userId], $[outdated]) ON CONFLICT (user_id) DO UPDATE SET outdated = $[outdated]";
        this.userSelect = "SELECT user_id, outdated FROM users WHERE user_id = $[userId]";
        this.userDeviceUpsert = "INSERT INTO user_devices (user_id, device_id, device, active) VALUES ($[userId], $[deviceId], $[device], $[active]) ON CONFLICT (user_id, device_id) DO UPDATE SET device = $[device], active = $[active]";
        this.userDevicesDelete = "UPDATE user_devices SET active = false WHERE user_id = $[userId]";
        this.userDevicesSelect = "SELECT user_id, device_id, device, active FROM user_devices WHERE user_id = $[userId]";
        this.userActiveDevicesSelect = "SELECT user_id, device_id, device, active FROM user_devices WHERE user_id = $[userId] AND active = true";
        this.userActiveDeviceSelect = "SELECT user_id, device_id, device, active FROM user_devices WHERE user_id = $[userId] AND device_id = $[deviceId] AND active = true";
        this.obGroupSessionUpsert = "INSERT INTO outbound_group_sessions (ns, session_id, room_id, current, pickled, uses_left, expires_ts) VALUES ($[ns], $[sessionId], $[roomId], $[current], $[pickled], $[usesLeft], $[expiresTs]) ON CONFLICT (ns, session_id, room_id) DO UPDATE SET pickled = $[pickled], current = $[current], uses_left = $[usesLeft], expires_ts = $[expiresTs]";
        this.obGroupSessionSelect = "SELECT ns, session_id, room_id, current, pickled, uses_left, expires_ts FROM outbound_group_sessions WHERE session_id = $[sessionId] AND room_id = $[roomId] AND ns = $[ns]";
        this.obGroupCurrentSessionSelect = "SELECT ns, session_id, room_id, current, pickled, uses_left, expires_ts FROM outbound_group_sessions WHERE room_id = $[roomId] AND current = true AND ns = $[ns]";
        this.obGroupSessionMarkAllInactive = "UPDATE outbound_group_sessions SET current = false WHERE room_id = $[roomId] AND ns = $[ns]";
        this.obSentGroupSessionUpsert = "INSERT INTO sent_outbound_group_sessions (ns, session_id, room_id, session_index, user_id, device_id) VALUES ($[ns], $[sessionId], $[roomId], $[sessionIndex], $[userId], $[deviceId]) ON CONFLICT (ns, session_id, room_id, user_id, device_id, session_index) DO NOTHING";
        this.obSentSelectLastSent = "SELECT ns, session_id, room_id, session_index, user_id, device_id FROM sent_outbound_group_sessions WHERE user_id = $[userId] AND device_id = $[deviceId] AND room_id = $[roomId] AND ns = $[ns]";
        this.olmSessionUpsert = "INSERT INTO olm_sessions (ns, user_id, device_id, session_id, last_decryption_ts, pickled) VALUES ($[ns], $[userId], $[deviceId], $[sessionId], $[lastDecryptionTs], $[pickled]) ON CONFLICT (ns, user_id, device_id, session_id) DO UPDATE SET last_decryption_ts = $[lastDecryptionTs], pickled = $[pickled]";
        this.olmSessionCurrentSelect = "SELECT ns, user_id, device_id, session_id, last_decryption_ts, pickled FROM olm_sessions WHERE user_id = $[userId] AND device_id = $[deviceId] AND ns = $[ns] ORDER BY last_decryption_ts DESC LIMIT 1";
        this.olmSessionSelect = "SELECT ns, user_id, device_id, session_id, last_decryption_ts, pickled FROM olm_sessions WHERE user_id = $[userId] AND device_id = $[deviceId] AND ns = $[ns]";
        this.ibGroupSessionUpsert = "INSERT INTO inbound_group_sessions (ns, session_id, room_id, user_id, device_id, pickled) VALUES ($[ns], $[sessionId], $[roomId], $[userId], $[deviceId], $[pickled]) ON CONFLICT (ns, session_id, room_id, user_id, device_id) DO UPDATE SET pickled = $[pickled]";
        this.ibGroupSessionSelect = "SELECT ns, session_id, room_id, user_id, device_id, pickled FROM inbound_group_sessions WHERE session_id = $[sessionId] AND room_id = $[roomId] AND user_id = $[userId] AND device_id = $[deviceId] AND ns = $[ns]";
        this.deMetadataUpsert = "INSERT INTO decrypted_event_metadata (ns, room_id, event_id, session_id, message_index) VALUES ($[ns], $[roomId], $[eventId], $[sessionId], $[messageIndex]) ON CONFLICT (ns, room_id, event_id) DO UPDATE SET message_index = $[messageIndex], session_id = $[sessionId]";
        this.deMetadataSelect = "SELECT ns, room_id, event_id, session_id, message_index FROM decrypted_event_metadata WHERE room_id = $[roomId] AND session_id = $[sessionId] AND message_index = $[messageIndex] AND ns = $[ns] LIMIT 1";
        if (parentOrSecureStorage instanceof NamespacingPostgresCryptoStorageProvider) {
            this.namespace = cnOrNamespace;
            this.setup = parentOrSecureStorage.setup;
            this.db = parentOrSecureStorage.db;
            this.secureStorage = parentOrSecureStorage.secureStorage;
            return;
        }
        this.secureStorage = parentOrSecureStorage;
        this.db = psql(cnOrNamespace);
        this.setup = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.db.none("CREATE TABLE IF NOT EXISTS kv (ns TEXT NOT NULL, name TEXT NOT NULL, value TEXT NOT NULL, PRIMARY KEY (ns, name))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS rooms (room_id TEXT PRIMARY KEY NOT NULL, config TEXT NOT NULL)");
                yield this.db.none("CREATE TABLE IF NOT EXISTS users (user_id TEXT PRIMARY KEY NOT NULL, outdated BOOL NOT NULL)");
                yield this.db.none("CREATE TABLE IF NOT EXISTS user_devices (user_id TEXT NOT NULL, device_id TEXT NOT NULL, device TEXT NOT NULL, active BOOL NOT NULL, PRIMARY KEY (user_id, device_id))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS outbound_group_sessions (ns TEXT NOT NULL, session_id TEXT NOT NULL, room_id TEXT NOT NULL, current BOOL NOT NULL, pickled TEXT NOT NULL, uses_left INT NOT NULL, expires_ts BIGINT NOT NULL, PRIMARY KEY (ns, session_id, room_id))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS sent_outbound_group_sessions (ns TEXT NOT NULL, session_id TEXT NOT NULL, room_id TEXT NOT NULL, session_index INT NOT NULL, user_id TEXT NOT NULL, device_id TEXT NOT NULL, PRIMARY KEY (ns, session_id, room_id, user_id, device_id, session_index))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS olm_sessions (ns TEXT NOT NULL, user_id TEXT NOT NULL, device_id TEXT NOT NULL, session_id TEXT NOT NULL, last_decryption_ts BIGINT NOT NULL, pickled TEXT NOT NULL, PRIMARY KEY (ns, user_id, device_id, session_id))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS inbound_group_sessions (ns TEXT NOT NULL, session_id TEXT NOT NULL, room_id TEXT NOT NULL, user_id TEXT NOT NULL, device_id TEXT NOT NULL, pickled TEXT NOT NULL, PRIMARY KEY (ns, session_id, room_id, user_id, device_id))");
                yield this.db.none("CREATE TABLE IF NOT EXISTS decrypted_event_metadata (ns TEXT NOT NULL, room_id TEXT NOT NULL, event_id TEXT NOT NULL, session_id TEXT NOT NULL, message_index INT NOT NULL, PRIMARY KEY (ns, room_id, event_id))");
                yield this.db.none("CREATE INDEX IF NOT EXISTS idx_decrypted_event_metadata_by_message_index ON decrypted_event_metadata (ns, room_id, session_id, message_index)");
                resolve();
            }
            catch (e) {
                reject(e);
            }
        }));
    }
    setDeviceId(deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.kvUpsert, {
                ns: this.namespace,
                name: 'deviceId',
                value: deviceId,
            });
        });
    }
    getDeviceId() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const row = yield this.db.oneOrNone(this.kvSelect, {
                ns: this.namespace,
                name: 'deviceId',
            });
            return row === null || row === void 0 ? void 0 : row.value;
        });
    }
    setPickleKey(pickleKey) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.secureStorage.setPickleKey(pickleKey);
        });
    }
    getPickleKey() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.secureStorage.getPickleKey();
        });
    }
    setPickledAccount(pickled) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            // TODO: Use secure storage
            return this.db.none(this.kvUpsert, {
                ns: this.namespace,
                name: 'pickled',
                value: pickled,
            });
        });
    }
    getPickledAccount() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const row = yield this.db.oneOrNone(this.kvSelect, {
                ns: this.namespace,
                name: 'pickled',
            });
            return row === null || row === void 0 ? void 0 : row.value;
        });
    }
    storeRoom(roomId, config) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.roomUpsert, {
                roomId: roomId,
                config: JSON.stringify(config),
            });
        });
    }
    getRoom(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const row = yield this.db.oneOrNone(this.roomSelect, {
                roomId: roomId,
            });
            const val = row === null || row === void 0 ? void 0 : row.config;
            return val ? JSON.parse(val) : null;
        });
    }
    setActiveUserDevices(userId, devices) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.tx((t) => __awaiter(this, void 0, void 0, function* () {
                yield t.none(this.userUpsert, {
                    userId: userId,
                    outdated: false,
                });
                yield t.none(this.userDevicesDelete, {
                    userId: userId,
                });
                for (const device of devices) {
                    yield t.none(this.userDeviceUpsert, {
                        userId: userId,
                        deviceId: device.device_id,
                        device: JSON.stringify(device),
                        active: true,
                    });
                }
            }));
        });
    }
    getActiveUserDevices(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const results = yield this.db.manyOrNone(this.userActiveDevicesSelect, {
                userId: userId,
            });
            if (!results)
                return [];
            return results.map(d => JSON.parse(d.device));
        });
    }
    getActiveUserDevice(userId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.userActiveDeviceSelect, {
                userId: userId,
                deviceId: deviceId,
            });
            return result ? JSON.parse(result.device) : null;
        });
    }
    getAllUserDevices(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const results = yield this.db.manyOrNone(this.userDevicesSelect, {
                userId: userId,
            });
            if (!results)
                return [];
            return results.map(d => Object.assign({}, JSON.parse(d.device), { unsigned: { bsdkIsActive: d.active } }));
        });
    }
    flagUsersOutdated(userIds) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.tx((t) => __awaiter(this, void 0, void 0, function* () {
                for (const userId of userIds) {
                    yield t.none(this.userUpsert, {
                        userId: userId,
                        outdated: true,
                    });
                }
            }));
        });
    }
    isUserOutdated(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const user = yield this.db.oneOrNone(this.userSelect, {
                userId: userId,
            });
            return user ? user.outdated : true;
        });
    }
    storeOutboundGroupSession(session) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.tx((t) => __awaiter(this, void 0, void 0, function* () {
                if (session.isCurrent) {
                    yield t.none(this.obGroupSessionMarkAllInactive, {
                        ns: this.namespace,
                        roomId: session.roomId,
                    });
                }
                yield t.none(this.obGroupSessionUpsert, {
                    ns: this.namespace,
                    sessionId: session.sessionId,
                    roomId: session.roomId,
                    pickled: session.pickled,
                    current: session.isCurrent,
                    usesLeft: session.usesLeft,
                    expiresTs: session.expiresTs,
                });
            }));
        });
    }
    getOutboundGroupSession(sessionId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.obGroupSessionSelect, {
                ns: this.namespace,
                sessionId: sessionId,
                roomId: roomId,
            });
            if (result) {
                return {
                    sessionId: result.session_id,
                    roomId: result.room_id,
                    pickled: result.pickled,
                    isCurrent: result.current,
                    usesLeft: result.uses_left,
                    expiresTs: result.expires_ts,
                };
            }
            return null;
        });
    }
    getCurrentOutboundGroupSession(roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.obGroupCurrentSessionSelect, {
                ns: this.namespace,
                roomId: roomId,
            });
            if (result) {
                return {
                    sessionId: result.session_id,
                    roomId: result.room_id,
                    pickled: result.pickled,
                    isCurrent: result.current,
                    usesLeft: result.uses_left,
                    expiresTs: result.expires_ts,
                };
            }
            return null;
        });
    }
    storeSentOutboundGroupSession(session, index, device) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.obSentGroupSessionUpsert, {
                ns: this.namespace,
                sessionId: session.sessionId,
                roomId: session.roomId,
                sessionIndex: index,
                userId: device.user_id,
                deviceId: device.device_id,
            });
        });
    }
    getLastSentOutboundGroupSession(userId, deviceId, roomId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.obSentSelectLastSent, {
                ns: this.namespace,
                userId: userId,
                deviceId: deviceId,
                roomId: roomId,
            });
            if (result) {
                return { sessionId: result.session_id, index: result.session_index };
            }
            return null;
        });
    }
    storeOlmSession(userId, deviceId, session) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.olmSessionUpsert, {
                ns: this.namespace,
                userId: userId,
                deviceId: deviceId,
                sessionId: session.sessionId,
                lastDecryptionTs: session.lastDecryptionTs,
                pickled: session.pickled,
            });
        });
    }
    getCurrentOlmSession(userId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.olmSessionCurrentSelect, {
                ns: this.namespace,
                userId: userId,
                deviceId: deviceId,
            });
            if (!result)
                return null;
            return {
                sessionId: result.session_id,
                pickled: result.pickled,
                lastDecryptionTs: result.last_decryption_ts,
            };
        });
    }
    getOlmSessions(userId, deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.manyOrNone(this.olmSessionSelect, {
                ns: this.namespace,
                userId: userId,
                deviceId: deviceId,
            });
            return (result || []).map(r => ({
                sessionId: r.session_id,
                pickled: r.pickled,
                lastDecryptionTs: r.last_decryption_ts,
            }));
        });
    }
    storeInboundGroupSession(session) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.ibGroupSessionUpsert, {
                ns: this.namespace,
                sessionId: session.sessionId,
                roomId: session.roomId,
                userId: session.senderUserId,
                deviceId: session.senderDeviceId,
                pickled: session.pickled,
            });
        });
    }
    getInboundGroupSession(senderUserId, senderDeviceId, roomId, sessionId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.ibGroupSessionSelect, {
                ns: this.namespace,
                sessionId: sessionId,
                roomId: roomId,
                userId: senderUserId,
                deviceId: senderDeviceId,
            });
            if (result) {
                return {
                    sessionId: result.session_id,
                    roomId: result.room_id,
                    senderUserId: result.user_id,
                    senderDeviceId: result.device_id,
                    pickled: result.pickled,
                };
            }
            return null;
        });
    }
    setMessageIndexForEvent(roomId, eventId, sessionId, messageIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            return this.db.none(this.deMetadataUpsert, {
                ns: this.namespace,
                roomId: roomId,
                eventId: eventId,
                sessionId: sessionId,
                messageIndex: messageIndex,
            });
        });
    }
    getEventForMessageIndex(roomId, sessionId, messageIndex) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setup;
            const result = yield this.db.oneOrNone(this.deMetadataSelect, {
                ns: this.namespace,
                roomId: roomId,
                sessionId: sessionId,
                messageIndex: messageIndex,
            });
            return result === null || result === void 0 ? void 0 : result.event_id;
        });
    }
    storageForUser(userId) {
        return new NamespacingPostgresCryptoStorageProvider(userId, this);
    }
}
exports.NamespacingPostgresCryptoStorageProvider = NamespacingPostgresCryptoStorageProvider;
