"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.List = exports.ListBehaviour = void 0;
const MembershipEvent_1 = require("../models/events/MembershipEvent");
var ListBehaviour;
(function (ListBehaviour) {
    ListBehaviour[ListBehaviour["General"] = 0] = "General";
    ListBehaviour[ListBehaviour["Invites"] = 1] = "Invites";
})(ListBehaviour = exports.ListBehaviour || (exports.ListBehaviour = {}));
class List {
    // TODO: Handle range requests
    // TODO: Do `this.processEvent` from MatrixClient
    constructor(listIndex, client, behaviour) {
        this.listIndex = listIndex;
        this.client = client;
        this.behaviour = behaviour;
        this.roomIds = [];
        // noinspection JSIgnoredPromiseFromCall
        this.loadList();
    }
    get orderedRoomIds() {
        return this.roomIds.map(r => r); // clone to keep immutability
    }
    handleSyncOp(op) {
        return __awaiter(this, void 0, void 0, function* () {
            this.roomIds = op.rooms.map(r => r.room_id);
            while (this.roomIds.length <= (op.range[1] - op.range[0])) {
                this.roomIds.push(null);
            }
            yield this.storeList();
            for (const room of op.rooms) {
                yield this.handleTimeline(room.room_id, room.timeline);
            }
        });
    }
    handleInsertOp(op) {
        return __awaiter(this, void 0, void 0, function* () {
            this.roomIds.splice(op.index, 0, op.room.room_id);
            yield this.storeList();
            yield this.handleTimeline(op.room.room_id, op.room.timeline);
        });
    }
    handleUpdateOp(op) {
        return __awaiter(this, void 0, void 0, function* () {
            // Update in place - process potential timeline data
            if (!op.room.timeline)
                return; // nothing to do
            yield this.handleTimeline(this.roomIds[op.index], op.room.timeline);
        });
    }
    handleDeleteOp(op) {
        return __awaiter(this, void 0, void 0, function* () {
            this.roomIds.splice(op.index, 1);
            yield this.storeList();
        });
    }
    storeList() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.client.storageProvider.storeValue("sync_v3_list_" + this.listIndex, JSON.stringify(this.roomIds));
        });
    }
    loadList() {
        return __awaiter(this, void 0, void 0, function* () {
            this.roomIds = JSON.parse((yield this.client.storageProvider.readValue("sync_v3_list_" + this.listIndex)) || "[]");
        });
    }
    findMembership(timeline) {
        return __awaiter(this, void 0, void 0, function* () {
            const myUserId = yield this.client.getUserId();
            return timeline.slice().reverse().find(e => e.state_key === myUserId && e.type === "m.room.member");
        });
    }
    handleTimeline(roomId, timeline) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!timeline)
                return;
            // if (this.behaviour !== ListBehaviour.General) return; // TODO: See operation comment below
            // Ideally we'd use the operation to determine the membership transition based on the ListBehaviour,
            // but instead we're stuck with trying to figure it out from a timeline update on *all* operations.
            const rawMembership = yield this.findMembership(timeline);
            if (rawMembership) {
                const membership = new MembershipEvent_1.MembershipEvent(rawMembership);
                // TODO: Handle atomic updates.
                // If there's a UPDATE, DELETE, INSERT then we probably don't want to fire 2 membership updates
                if (membership.effectiveMembership === "join") {
                    this.client.emit("room.join", roomId, membership.raw);
                }
                else if (membership.effectiveMembership === "leave") {
                    this.client.emit("room.leave", roomId, membership.raw);
                }
                else if (membership.effectiveMembership === "invite") {
                    this.client.emit("room.invite", roomId, membership.raw);
                }
            }
            // TODO: Exclude events that were pre-join
            for (const event of timeline) {
                if (event['type'] === "m.room.message") {
                    this.client.emit("room.message", roomId, event);
                }
                else if (event['type'] === "m.room.tombstone" && event['state_key'] === '') {
                    this.client.emit("room.archived", roomId, event);
                }
                else if (event['type'] === "m.room.create" && event['state_key'] === '' && ((_b = (_a = event['content']) === null || _a === void 0 ? void 0 : _a['predecessor']) === null || _b === void 0 ? void 0 : _b['room_id'])) {
                    this.client.emit("room.upgraded", roomId, event);
                }
                this.client.emit("room.event", roomId, event);
            }
        });
    }
}
exports.List = List;
