import { MatrixClient } from "../MatrixClient";
import { IStorageProvider } from "../storage/IStorageProvider";
import { IV3List, SortBehaviour } from "./V3List";
/**
 * A MatrixClient class which attempts to use Sync V3 instead of the normal sync protocol
 * on the server.
 *
 * This class is considered <b>UNSTABLE</b> and may be removed at any time. This class will
 * not check for server support before attempting to use it.
 *
 * @category Unstable: Sync V3
 */
export declare class SyncV3MatrixClient extends MatrixClient {
    private lastPos;
    private lists;
    /**
     * Creates a new matrix client
     * @param {string} homeserverUrl The homeserver's client-server API URL
     * @param {string} accessToken The access token for the homeserver
     * @param {IStorageProvider} storage The storage provider to use.
     * @param {SortBehaviour[]} sortBehaviours The list sorting behaviour to use.
     */
    constructor(homeserverUrl: string, accessToken: string, storage: IStorageProvider, sortBehaviours?: SortBehaviour[]);
    get joinedList(): IV3List;
    get invitedList(): IV3List;
    get directMessagesList(): IV3List;
    protected startSyncInternal(): Promise<any>;
    private loopSync;
    private processSyncV3;
    private doSyncV3;
}
