import { IV3Room, V3Room } from "./V3Room";
import { OpSync, SyncV3Operation } from "./operations";
import { SyncV3List } from "./models";
import { MatrixClient } from "../MatrixClient";
/**
 * @category Unstable: Sync V3
 */
export declare enum ListBehaviour {
    JoinedOnly = 0,
    InvitedOnly = 1,
    DMsOnly = 2
}
/**
 * @category Unstable: Sync V3
 */
export declare enum SortBehaviour {
    NotificationCount = "by_notification_count",
    Recency = "by_recency",
    Name = "by_name"
}
/**
 * @category Unstable: Sync V3
 */
export declare const CURRENT_USER_STATE_KEY = "@CURRENT";
/**
 * @category Unstable: Sync V3
 */
export declare type RequiredStateTuple = [string, string];
/**
 * @category Unstable: Sync V3
 */
export interface IV3List {
    totalRoomCount: number;
    orderedKnownRooms: IV3Room[];
    unorderedAllRooms: IV3Room[];
}
/**
 * @category Unstable: Sync V3
 */
export declare class V3List implements IV3List {
    readonly behaviour: ListBehaviour;
    readonly sort: SortBehaviour[];
    private roomsByRange;
    private roomMap;
    private totalCount;
    constructor(behaviour: ListBehaviour, sort: SortBehaviour[]);
    get lossySerialized(): Omit<OpSync, "list">[];
    get totalRoomCount(): number;
    get orderedKnownRooms(): V3Room[];
    get unorderedAllRooms(): V3Room[];
    private findRange;
    private findIndexInRange;
    getDefinitionFor(currentUserId: string): SyncV3List;
    private getOrCreateRoom;
    private getOrCreateRangeFromIndex;
    processOperations(client: MatrixClient, totalCount: number, ops: SyncV3Operation[]): Promise<void>;
}
