"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V3Room = void 0;
const RoomNameEvent_1 = require("../models/events/RoomNameEvent");
const RoomTopicEvent_1 = require("../models/events/RoomTopicEvent");
const EncryptionEvent_1 = require("../models/events/EncryptionEvent");
const V3List_1 = require("./V3List");
const MembershipEvent_1 = require("../models/events/MembershipEvent");
/**
 * @category Unstable: Sync V3
 */
class V3Room {
    // TODO: Include other event types as "required"?
    constructor(roomId) {
        this.roomId = roomId;
    }
    get lossySerialized() {
        var _a;
        return {
            room_id: this.roomId,
            timeline: [],
            required_state: [
                this.nameEvent, this.topicEvent, this.encryptionEvent, this.myMembership,
            ].filter(e => !!e).map(e => e.raw),
            name: (_a = this.nameEvent) === null || _a === void 0 ? void 0 : _a.name,
            // We don't know these values, but also don't really care
            highlight_count: 0,
            notification_count: 0,
        };
    }
    get name() {
        var _a, _b;
        return (_b = (_a = this.nameEvent) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : undefined;
    }
    get topic() {
        var _a, _b;
        return (_b = (_a = this.topicEvent) === null || _a === void 0 ? void 0 : _a.topic) !== null && _b !== void 0 ? _b : undefined;
    }
    get encryptionConfig() {
        var _a, _b, _c;
        if (!((_a = this.encryptionEvent) === null || _a === void 0 ? void 0 : _a.algorithm))
            return undefined;
        return (_c = (_b = this.encryptionEvent) === null || _b === void 0 ? void 0 : _b.content) !== null && _c !== void 0 ? _c : undefined;
    }
    get membership() {
        return this.myMembership;
    }
    updateState(client, state) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Send events through pre-processor
            for (const event of state) {
                if (event['type'] === "m.room.name" && event['state_key'] === '')
                    this.nameEvent = new RoomNameEvent_1.RoomNameEvent(event);
                if (event['type'] === "m.room.topic" && event['state_key'] === '')
                    this.topicEvent = new RoomTopicEvent_1.RoomTopicEvent(event);
                if (event['type'] === "m.room.encryption" && event['state_key'] === '')
                    this.encryptionEvent = new EncryptionEvent_1.EncryptionEvent(event);
                if (event['type'] === "m.room.member" && event['state_key'] === (yield client.getUserId()))
                    this.myMembership = new MembershipEvent_1.MembershipEvent(event);
            }
        });
    }
    updateTimeline(client, timeline) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: Send events through pre-processor
            // TODO: Process membership
            // TODO: Process events
            console.log('@@ TIMELINE', timeline);
        });
    }
    static get REQUIRED_STATE() {
        return [
            ["m.room.name", ""],
            ["m.room.topic", ""],
            ["m.room.encryption", ""],
            ["m.room.member", V3List_1.CURRENT_USER_STATE_KEY],
        ];
    }
}
exports.V3Room = V3Room;
