"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CryptexCryptoSecureStorageProvider = void 0;
const cryptex_1 = require("cryptex");
/**
 * A storage provider for secure information relating to crypto, stored using Cryptex.
 * Requires `cryptex` package to be installed.
 *
 * The pickle key is stored as the "pickle_key" secret.
 * @category Storage providers
 * @see https://github.com/TomFrost/Cryptex
 */
class CryptexCryptoSecureStorageProvider {
    constructor(opts) {
        this.cryptex = new cryptex_1.Cryptex(opts);
    }
    getPickleKey() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.cryptex.getSecret("pickle_key");
        });
    }
    setPickleKey(pickleKey) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("Cannot set keys with Cryptex - ensure that the secret is set up prior to running");
        });
    }
}
exports.CryptexCryptoSecureStorageProvider = CryptexCryptoSecureStorageProvider;
